/*
**  BundleLoader.m
**
**  Copyright (c) 2003
**
**  Author: Yen-Ju  <yjchenx@hotmail.com>
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "BundleLoader.h"
#include <Foundation/NSString.h>
#include <Foundation/NSArray.h>
#include <Foundation/NSFileManager.h>
#include <Foundation/NSBundle.h>
#include <Foundation/NSPathUtilities.h>

static BundleLoader *defaultLoader = nil;

NSString *ToolboxLocalLibraryPath()
{
  NSString *string;

  string = [NSString stringWithFormat: @"%@/Toolbox", 
                      [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSLocalDomainMask, YES)
                                                          objectAtIndex:0] ];

  return string;
}

@implementation BundleLoader

+ (id) defaultLoader
{
  if ( !defaultLoader)
    {
      defaultLoader = [[BundleLoader alloc] init];
    }
  return defaultLoader;
}

- (id) init
{
  self = [super init];
  return self;
}

- (void) dealloc
{
  [super dealloc];
}

//
// Return all bundles name with specific extension
// in $GNUSTEP_USER_ROOT/Library/Toolbox
// The bundle name is the path of the bundle
//
- (NSArray *) allNamesOfBundles
{
  NSFileManager *aFileManager;
  NSArray *allFiles;
  NSMutableArray *allNames;
  NSString *extension = @"toolbox";
  int i;

  aFileManager = [NSFileManager defaultManager];
  allNames = [NSMutableArray new];

  /* Search for ~/GNUstep/Library/Toolbox/ */
  allFiles = [aFileManager directoryContentsAtPath: ToolboxLocalLibraryPath()];

  for (i = 0; i < [allFiles count]; i++)
    {
      NSString *aString;
      
      aString = [allFiles objectAtIndex: i];
      
      // If we found a bundle and the extension is equal, then load it!
      if ( [[aString pathExtension] isEqualToString: extension] )
        {
          [allNames addObject: aString];
        }
    }

  return AUTORELEASE(allNames);
}

// Return the Class name of a specific bundle
- (Class) classNameOf: (NSString *) bundleName
{
  NSBundle *bundle;
  Class bundleClass;
  NSString *path;

  /* Try ~/GNUstep/Library/Toolbox */
  path = [NSString stringWithFormat: @"%@/%@",
                                     ToolboxLocalLibraryPath(),
                                     bundleName];
  
  bundle = [NSBundle bundleWithPath: path];

  if (bundle)
    {
      bundleClass = [bundle principalClass];

      if ( bundleClass )
        {
          return bundleClass;
        }
      else
        {
          NSLog(@"Failed to get Class from bundle %@", bundleName);
          return nil;
        }
    }

  NSLog(@"BundleLoader: Error loading bundle at path %@", bundleName);
  return nil;
}

// Return the standard resource of a specific bundle
- (NSString *) pathForResource: (NSString *) filename ofType: (NSString *) type
              ofBundle: (NSString *) bundleName
{
  NSBundle *bundle;
  NSString *path;

  /* Try ~/GNUstep/Library/Toolbox */
  path = [NSString stringWithFormat: @"%@/%@",
                                     ToolboxLocalLibraryPath(),
                                     bundleName];

  bundle = [NSBundle bundleWithPath: path];

  if (bundle)
    {
      return [bundle pathForResource: filename ofType: type];
    }

  NSLog(@"BundleLoader: Error loading bundle at path %@", bundleName);
  return nil;
}

@end
